
/*  -*- MaTX -*-
 *
 *  yOz
 *      cov() - U
 *
 *  y`z
 *      C = cov(X)
 *         Matrix C;
 *         Matrix X;
 *
 *      C = cov(X,Y)
 *         Matrix C;
 *         Matrix X,Y;
 *
 *  y@\z
 *      cov(X) X ̑SĂ̐̕UԂB
 *      cov(X,Y) ͍s [X Y] ̕UԂB
 *
 *  yz
 *      >> cov([1 2 3 4 5])
 *      ans = 2.5
 *      >> cov([1 2 3 4 5], [6 7 8 9 10])
 *      ans = 9.16667
 *
 *  y֘Aځz
 *      sum, median, mean, cov_row, cov_col
 *
 *   -----------------------------------------------------------------------
 *
 *  yOz
 *      cov_col() - Us(񖈂̃f[^)
 *
 *  y`z
 *      C = cov_col(X)
 *         Matrix C;
 *         Matrix X;
 *
 *      C = cov_col(X,Y)
 *         Matrix C;
 *         Matrix X,Y;
 *
 *  y@\z
 *      X xNĝƂCcov_col(X) X ̑SĂ̐̕UԂB
 *      X ŝƂCcov_col(X) ́CX ̗񖈂Ƀf[^ۑ
 *      ƌȂCUs߂B
 *
 *      cov_col(X,Y) ́CX  Y ̃xNgȂC
 *      cov_col([makecolv(X) makecolv(Y)])ƓłB
 *
 *  yz
 *      >> C = cov_col([1 2 3 4 5])
 *      ===  C  (   1  x   1)  Matrix  ===
 *                 (  1)     
 *      (  1)  2.50000000E+00
 *      >> C = cov_col([2 3 1 4 5], [10 9 7 8 6])
 *      === [C] : (  2,  2) ===
 *                 (  1)           (  2)     
 *      (  1)  2.50000000E+00 -1.00000000E+00
 *      (  2) -1.00000000E+00  2.50000000E+00
 *
 *  y֘Aځz
 *      sum, median, mean, cov, cov_row
 *
 *   ------------------------------------------------------------------------
 *
 *  yOz
 *      cov_row() - Us(s̃f[^)
 *
 *  y`z
 *      C = cov_row(X)
 *         Matrix C;
 *         Matrix X;
 *
 *      C = cov_row(X,Y)
 *         Matrix C;
 *         Matrix X,Y;
 *
 *  y@\z
 *      X xNĝƂCcov_row(X) X ̑SĂ̐̕UԂB
 *      X ŝƂCcov_row(X) ́CX ̍sɃf[^ۑ
 *      ƌȂCUs߂B
 *
 *      cov_row(X,Y) ́CX  Y ̃xNgȂC
 *      cov_row([[makerowv(X)][makerowv(Y)]])ƓłB
 *
 *  yz
 *      >> C = cov_row([1 2 3 4 5])
 *      ===  C  (   1  x   1)  Matrix  ===
 *                 (  1)     
 *      (  1)  2.50000000E+00
 *      >> C = cov_row([2 3 1 4 5], [10 9 7 8 6])
 *      ===  C  (   2  x   2)  Matrix  ===
 *                 (  1)           (  2)     
 *      (  1)  2.50000000E+00 -1.00000000E+00
 *      (  2) -1.00000000E+00  2.50000000E+00
 *
 *  y֘Aځz
 *      sum, median, mean, cov, cov_col
 */


Func Real cov(x, y, ...)
	Matrix x, y;
{
	Matrix z;

	error(nargchk(1, 2, nargs, "cov"));
	
	if (nargs == 1) {
		return [cov_col(reshape(x, 1, Rows(x)*Cols(x)))](1,1);
	} else {
		z = [makecolv(x), makecolv(y)];
		return [cov_col(reshape(z, 1, Rows(z)*Cols(z)))](1,1);
	}
}

Func Matrix cov_col(x_, y_, ...)
	Matrix x_, y_;
{
	Integer m, n;
	Matrix X, cv;

	error(nargchk(1, 2, nargs, "cov_col"));

	{m,n} = size(x_);

	if (nargs > 1) {
		if (m != Rows(y_) || n != Cols(y_)) {
			error("cov_col(): X  Y ͓傫łȂ΂Ȃ炢B\n");
		}
		if (m != 1 && n != 1) {
			error("cov_col(): X  Y ̓xNgłȂ΂Ȃ炢B\n");
		}
		X = [makecolv(x_), makecolv(y_)];
	} else if (min(Cols(x_), Rows(x_)) == 1) {
		X = makecolv(x_);
	} else {
		X = x_;
	}

	m = Rows(X);
	X = X - ONE(m,1) * (sum_col(X)/m);

	if (m == 1) {
		cv = Z(1);
	} else {
		cv = X#*X / (m-1);
	}
	return cv;
}

Func Matrix cov_row(x, y, ...)
	Matrix x, y;
{
	error(nargchk(1, 2, nargs, "cov_row"));
	
	if (nargs == 1) {
		return trans(cov_col(trans(x)));
	} else {
		return trans(cov_col(x, y));
	}
}
